import sqlite3
from loguru import logger


import threatingestor.exceptions


class State:
    """State DB management."""
    def __init__(self, dbname):
        """Set up a connection to the state DB."""
        try:
            self.conn = sqlite3.connect(dbname)
            self.cursor = self.conn.cursor()
            self._create_table()
        except sqlite3.Error:
            raise threatingestor.exceptions.IngestorError("State database seems broken")


    def _create_table(self):
        """Create table if it doesn't already exist."""
        self.cursor.execute('CREATE TABLE IF NOT EXISTS states (name text UNIQUE, state text)')
        self.conn.commit()


    def save_state(self, name, state):
        """Create or update a state record."""
        logger.debug(f"Updating state for '{name}' to '{state}'")
        self.cursor.execute('INSERT OR REPLACE INTO states (name, state) values (?, ?)', (name, state))
        self.conn.commit()


    def get_state(self, name):
        """Return the state string for a given plugin."""
        logger.debug(f"Getting state for '{name}'")
        self.cursor.execute('SELECT state FROM states WHERE name=?', (name,))
        res = self.cursor.fetchone()
        return res[0] if res else res



import sys


import requests


import threatingestor.extras.queueworker


class PasteProcessor(threatingestor.extras.queueworker.QueueWorker):
    """Read pastebin URLs from a queue, write raw content to a queue."""

    def do_work(self, job):
        """From a paste URL, get the raw contents."""
        try:
            url = job['url']

        except KeyError:
            print(f"Bad job: {job}")
            return

        # Pastebin.com
        if url.startswith("https://pastebin.com/raw/"):
            pass

        elif url.startswith("https://pastebin.com/"):
            start, end = url.split['.com/']
            url = '.com/raw/'.join([start, end])

        # Gist
        elif url.startswith("https://gist.github.com/") and not 'raw' in url:
            url = url + '/raw'

        # Fetch and return.
        response = requests.get(url)
        return {
            'content': response.content,
            'reference': response.url,
        }


if __name__ == "__main__":
    worker = PasteProcessor()
    worker.read_config(sys.argv[1])
    worker.run_forever()



import datetime
import urllib.request
from urllib.parse import urlparse
from bs4 import BeautifulSoup

from threatingestor.sources import Source

class Plugin(Source):
    
    def __init__(self, name, url):
        self.name = name
        self.url = url

    def run(self, saved_state):
        saved_state = datetime.datetime.utcnow().isoformat()[:-7] + "Z"

        # Configures sitemap parsing
        response = urllib.request.urlopen(self.url)
        xml = BeautifulSoup(response, "lxml-xml", from_encoding=response.info().get_param("charset"))

        sitemapindex = xml.find_all("sitemapindex")
        sitemap = xml.find_all("urlset")

        if sitemapindex:
            get_sitemap_type = "sitemapindex"
        elif sitemap:
            get_sitemap_type = "urlset"
        
        sitemaps = xml.find_all("sitemap")

        get_child_sitemaps = []

        for sitemap in sitemaps:
            get_child_sitemaps.append(sitemap.findNext("loc").text)

        if get_sitemap_type == "sitemapindex":
            sitemaps = get_child_sitemaps
        else:
            sitemaps = [self.url]

        urls = xml.find_all("url")
        artifacts = []

        for sitemap in sitemaps:
            for url in urls:

                # Extracts only the 'loc' tag from the xml
                if xml.find("loc"):
                    loc = url.findNext("loc").text
                    parsed_uri = urlparse(loc)
                    domain = "{uri.netloc}".format(uri=parsed_uri)
                else:
                    loc = ""
                    domain = ""

                row = {
                    "domain": domain,
                    "loc": loc
                }

                # Locates all blog links within the sitemap
                if "blog" in row["loc"]:
                    print(row["loc"])
                    artifacts += self.process_element(row["loc"], self.url)
        
        return saved_state, artifacts


import datetime, requests

from threatingestor.sources import Source
import threatingestor.artifacts

REPO_SEARCH_URL = "https://api.github.com/search/repositories"

class Plugin(Source):
    """Github Source Plugin"""
    def __init__(self, name, search, num_of_days=10, username="", token=""):
        self.name = name
        self.search = search
        self.num_of_days = num_of_days

        if username and token:
            self.auth = (username, token)
        else:
            self.auth = None

    def _repository_search(self, params):
        """Returns a list of repository results."""
        
        # Iterates through pages of results from query.
        response = requests.get(REPO_SEARCH_URL, params=params, auth=self.auth)

        repo_list = []

        while True:
            for repo in response.json().get('items', []):
                repo_list.append(repo)

            if not response.links.get('next'):
                break

            response = requests.get(
                response.links.get('next')["url"], auth=self.auth)

        return repo_list

    def run(self, saved_state):
        """Returns a list of artifacts and the saved state"""
        # If no saved_state, search max 1 day ago.
        if not saved_state:
            saved_state = (datetime.datetime.utcnow() - datetime.timedelta(days=self.num_of_days)).isoformat()[:-7] + 'Z'

        params = {
            'q': "{search} created:>={timestamp}".format(
                search=self.search,
                timestamp=saved_state),
            "per_page": "100"
        }

        saved_state = datetime.datetime.utcnow().isoformat()[:-7] + 'Z'
        repo_list = self._repository_search(params)

        artifact_list = []
        
        for repo in repo_list:
            title = "Manual Task: GitHub {u}".format(u=repo['full_name'])
            description = 'URL: {u}\nTask autogenerated by ThreatIngestor from source: {s}'
            description = description.format(s=self.name, u=repo['html_url'])
            artifact = threatingestor.artifacts.Task(title, self.name, reference_link=repo['html_url'], reference_text=description)

            artifact_list.append(artifact)

        return saved_state, artifact_list



import json


from threatingestor.exceptions import DependencyError
from threatingestor.sources import abstract_json


try:
    import boto3
except ImportError:
    raise DependencyError("Dependency boto3 required for SQS operator is not installed")


class Plugin(abstract_json.AbstractPlugin):
    """Source for Amazon SQS."""
    def __init__(self, name, aws_access_key_id, aws_secret_access_key,
            aws_region, queue_name, paths, reference=None):
        """SQS source."""
        super(Plugin, self).__init__(name, paths, reference)
        self.sqs = boto3.resource('sqs', region_name=aws_region, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key)
        self.queue = self.sqs.get_queue_by_name(QueueName=queue_name)


    def get_objects(self, saved_state):
        content_list = []
        for message in self.queue.receive_messages():
            # Process a message.
            job = json.loads(message.body)
            content_list.append(job)

            # Let the queue know that the message is processed.
            message.delete()

        return saved_state, content_list



import datetime, requests

from threatingestor.sources import Source
import threatingestor.artifacts

def user_set(user):
    GIST_SEARCH_URL = "https://api.github.com/users/{0}/gists".format(user)
    return GIST_SEARCH_URL

class Plugin(Source):
    """Github Gist Source Plugin"""
    def __init__(self, name, user="", username="", token=""):
        self.name = name
        self.user = user

        if username and token:
            self.auth = (username, token)
        else:
            self.auth = None

    def _gist_search(self, params):
        """Returns a list of gist results."""
        
        # Iterates through pages of results from query.
        response = requests.get(user_set(self.user), params=params, auth=self.auth)

        gist_list = []

        for gist in response.json():
            gist_list.append(gist)

        return gist_list

    def run(self, saved_state):
        """Returns a list of artifacts and the saved state"""

        params = { "per_page": "100" }

        saved_state = datetime.datetime.utcnow().isoformat()[:-7] + 'Z'
        gist_list = self._gist_search(params)

        artifact_list = []
        
        for gist in gist_list:
            title = "Gist Owner: {0}".format(self.user)
            description = 'URL: {u}\nTask autogenerated by ThreatIngestor from source: {s}'
            description = description.format(s=self.name, u=gist['html_url'])
            artifact = threatingestor.artifacts.Task(title, self.name, reference_link=gist['html_url'], reference_text=description)

            artifact_list.append(artifact)

        return saved_state, artifact_list


from __future__ import absolute_import


import threatingestor.artifacts
from threatingestor.operators import Operator
from threatingestor.exceptions import DependencyError


try:
    import threatkb
except ImportError:
    raise DependencyError("Dependency threatkb required for ThreatKB operator is not installed")


class Plugin(Operator):
    """Operator for InQuest ThreatKB."""
    def __init__(self, url, token, secret_key, state, artifact_types=None, filter_string=None, allowed_sources=None, use_https=False):
        """ThreatKB operator."""
        self.state = state
        self.api = threatkb.ThreatKB(url, token, secret_key, use_https=use_https)

        super(Plugin, self).__init__(artifact_types, filter_string, allowed_sources)
        self.artifact_types = artifact_types or [
            threatingestor.artifacts.Domain,
            threatingestor.artifacts.IPAddress,
            threatingestor.artifacts.YARASignature,
        ]


    def handle_artifact(self, artifact):
        """Operate on a single artifact."""
        if isinstance(artifact, threatingestor.artifacts.Domain):
            self.handle_domain(artifact)
        elif isinstance(artifact, threatingestor.artifacts.IPAddress):
            self.handle_ipaddress(artifact)
        elif isinstance(artifact, threatingestor.artifacts.YARASignature):
            self.handle_yarasignature(artifact)
        elif isinstance(artifact, threatingestor.artifacts.Task):
            self.handle_task(artifact)


    def handle_domain(self, domain):
        """Handle a single domain."""
        self.api.create('c2dns', {
                'domain_name': str(domain),
                'match_type': '',
                'description': '{l}\n\n{t}'.format(l=domain.reference_link, t=domain.reference_text),
                'references': '',
                'expiration_type': '',
                'state': {'state': self.state},
                'tags': [],
                'metadata_values': {},
            })


    def handle_ipaddress(self, ipaddress):
        """Handle a single IP address."""
        self.api.create('c2ips', {
                'ip': str(ipaddress),
                'references': '',
                'description': '{l}\n\n{t}'.format(l=ipaddress.reference_link, t=ipaddress.reference_text),
                'expiration_type': '',
                'state': {'state': self.state},
                'asn': '',
                'country': '',
                'city': '',
                'st': '',
                'tags': [],
                'metadata_values': {},
            })


    def handle_yarasignature(self, yarasignature):
        """Handle a single YARA signature."""
        self.api.create('import', {
                'autocommit': 1,
                'import_text': str(yarasignature),
                'shared_reference': yarasignature.reference_link,
                'shared_state': self.state,
                'extract_ip': False,
                'extract_dns': False,
            })


    def handle_task(self, task):
        """Handle a single Task."""
        self.api.create('tasks', {
                'title': str(task),
                'description': task.reference_text,
                'state': {'state': self.state},
                'final_artifact': '',
            })



from __future__ import absolute_import


import threatingestor.artifacts
from threatingestor.operators import Operator
from threatingestor.exceptions import DependencyError


try:
    import pymisp
except ImportError:
    raise DependencyError(
        "Dependency PyMISP required for MISP operator is not installed"
    )


class Plugin(Operator):
    """Operator for MISP."""

    def __init__(self, url, key, ssl=True, tags=None, artifact_types=None, filter_string=None, allowed_sources=None):
        """MISP operator."""
        self.api = pymisp.ExpandedPyMISP(url, key, ssl)
        if tags:
            self.tags = tags
        else:
            self.tags = ['type:OSINT']
        self.event_info = 'ThreatIngestor Event: {source_name}'

        super(Plugin, self).__init__(
            artifact_types, filter_string, allowed_sources
        )
        self.artifact_types = artifact_types or [
            threatingestor.artifacts.Domain,
            threatingestor.artifacts.Hash,
            threatingestor.artifacts.IPAddress,
            threatingestor.artifacts.URL,
            threatingestor.artifacts.YARASignature,
        ]

    def handle_artifact(self, artifact):
        """Operate on a single artifact."""
        event = self._find_or_create_event(artifact)

        if isinstance(artifact, threatingestor.artifacts.Domain):
            event = self.handle_domain(artifact, event=event)
        if isinstance(artifact, threatingestor.artifacts.Hash):
            event = self.handle_hash(artifact, event=event)
        elif isinstance(artifact, threatingestor.artifacts.IPAddress):
            event = self.handle_ipaddress(artifact, event=event)
        if isinstance(artifact, threatingestor.artifacts.URL):
            event = self.handle_url(artifact, event=event)
        elif isinstance(artifact, threatingestor.artifacts.YARASignature):
            event = self.handle_yarasignature(artifact, event=event)

        self._update_or_create_event(event)

    def _update_or_create_event(self, event):
        """Update or create an event for the artifact."""
        event_dict = event.to_dict()
        attributes = event_dict.get("Attribute", [])
        if not attributes:
            return
        # If an event doesn't have "date" field, it is not created int MISP
        if event_dict.get("date") is None:
            self.api.add_event(event)
        else:
            self.api.update_event(event)

    def _find_or_create_event(self, artifact):
        """Find or create an event for the artifact."""
        event = self._find_event(artifact)
        if event is not None:
            return event

        return self._create_event(artifact)

    def _find_event(self, artifact):
        """Find an event which has the same refetrence link, return an Event object."""
        events = self.api.search(
            "events",
            limit=1,
            type_attribute="link",
            value=artifact.reference_link,
            pythonify=True
        )
        if len(events) == 1:
            return events[0]

        return None

    def _create_event(self, artifact):
        """Create an event in MISP, return an Event object."""
        event = pymisp.MISPEvent()
        event.info = self.event_info.format(source_name=artifact.source_name)

        # Add tags.
        for tag in self.tags:
            event.add_tag(tag)

        # Add references.
        if artifact.reference_link != "":
            event.add_attribute("link", artifact.reference_link)
        if artifact.reference_text != "":
            event.add_attribute("text", artifact.reference_text)
        if artifact.source_name != "":
            event.add_attribute("other",  f'source:{artifact.source_name}')

        return event

    def handle_domain(self, domain, event: pymisp.MISPEvent):
        """Handle a single domain."""
        event.add_attribute("domain", str(domain))
        return event

    def handle_hash(self, hash_, event: pymisp.MISPEvent):
        """Handle a single hash."""
        if hash_.hash_type() == hash_.MD5:
            event.add_attribute("md5", str(hash_))
        elif hash_.hash_type() == hash_.SHA1:
            event.add_attribute("sha1", str(hash_))
        elif hash_.hash_type() == hash_.SHA256:
            event.add_attribute("sha256", str(hash_))
        return event

    def handle_ipaddress(self, ipaddress, event: pymisp.MISPEvent):
        """Handle a single IP address."""
        event.add_attribute("ip-dst", str(ipaddress))
        return event

    def handle_url(self, url, event: pymisp.MISPEvent):
        """Handle a single URL."""
        event.add_attribute("url", str(url))
        return event

    def handle_yarasignature(self, yarasignature, event: pymisp.MISPEvent):
        """Handle a single YARA signature."""
        event.add_attribute("yara", str(yarasignature))
        return event



import cv2
import pytesseract
import iocextract
import requests
import datetime
import os
import numpy as np

from threatingestor.sources import Source
import threatingestor.artifacts

class Plugin(Source):
    """
    Image text extraction using Google's OCR Tesseract engine and computer vision
    """

    def __init__(self, name, img=""):
        self.name = name
        self.img = img

        if "http" in img:
            with open("/tmp/data.png", "wb") as i:
                i.write(requests.get(str(self.img)).content)

    def run(self, saved_state):
        saved_state = datetime.datetime.utcnow().isoformat()[:-7] + "Z"

        if os.path.exists("/tmp/data.png"):
            data = cv2.imread("/tmp/data.png")
        else:
            data = cv2.imread(self.img)

        try:
            # Helps with preprocessing by converting to a grayscale
            grayscale_img = cv2.cvtColor(data, cv2.COLOR_BGR2GRAY)

            # Creates a binary image by using the proper threshold from cv
            binary_img = cv2.threshold(grayscale_img, 130, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)[1]

            # Inverts the binary
            invert_img = cv2.bitwise_not(binary_img)

            # Helps with cleanup
            noise_reduction = np.ones((2,2), np.uint8)
            process_iter = cv2.erode(invert_img, noise_reduction, iterations = 1)
            process_iter = cv2.dilate(process_iter, noise_reduction, iterations = 1)

            # Converts image data to a string
            img_data = pytesseract.image_to_string(process_iter)

            artifact_list = []

            title = "Image: {0}".format(self.img)
            description = 'URL: {u}\nTask autogenerated by ThreatIngestor from source: {s}'
            description = description.format(s=self.name, u=list(iocextract.extract_urls(img_data)))
            artifact = threatingestor.artifacts.Task(title, self.name, reference_link=str(list(iocextract.extract_urls(img_data))), reference_text=description)
            artifact_list.append(artifact)
                
        except cv2.error:
            raise FileNotFoundError

        return saved_state, artifact_list


import io
import importlib


import yaml
from loguru import logger


from threatingestor.exceptions import IngestorError, PluginError
import threatingestor.artifacts


SOURCE = 'threatingestor.sources'
OPERATOR = 'threatingestor.operators'

INTERNAL_OPTIONS = [
    'saved_state',
    'module',
    'credentials',
]

ARTIFACT_TYPES = 'artifact_types'
FILTER_STRING = 'filter'
ALLOWED_SOURCES = 'allowed_sources'
NAME = 'name'


class Config:
    """Config read/write operations, and convenience methods."""
    def __init__(self, filename):
        """Read a config file."""
        self.filename = filename
        with io.open(self.filename, 'r') as f:
            try:
                self.config = yaml.safe_load(f.read())
            except yaml.error.YAMLError:
                raise IngestorError("YAML error in config")


    @staticmethod
    def _load_plugin(plugin_type, plugin):
        """Returns plugin class or raises an exception.

        :raises: threatingestor.exceptions.PluginError
        """
        try:
            module = importlib.import_module('.'.join([plugin_type, plugin]))
            return module.Plugin
        except (ImportError, AttributeError):
            raise PluginError("No valid plugin '{p}' in '{t}'".format(p=plugin, t=plugin_type))


    def daemon(self):
        """Returns boolean, are we daemonizing?"""
        return self.config['general']['daemon']


    def state_path(self):
        """Returns path of state.db file."""
        return self.config['general']['state_path']


    def sleep(self):
        """Returns number of seconds to sleep between iterations, if daemonizing."""
        return self.config['general']['sleep']


    def statsd(self):
        """Returns statsd config dictionary."""
        return self.config.get('statsd', {})


    def notifiers(self):
        """Returns notifiers config dictionary."""
        return self.config.get('notifiers', {})


    def logging(self):
        """Returns logging config dictionary."""
        return self.config.get('logging', {})


    def credentials(self, credential_name):
        """Return a dictionary with the specified credentials."""
        for credential in self.config['credentials']:
            for key, value in credential.items():
                if key == NAME and value == credential_name:
                    return credential
        return {}


    def sources(self):
        """Return a list of (name, Source class, {kwargs}) tuples.

        :raises: threatingestor.exceptions.PluginError
        """
        sources = []

        for source in self.config['sources']:
            kwargs = {}
            for key, value in source.items():
                if key not in INTERNAL_OPTIONS:
                    kwargs[key] = value

                elif key == 'credentials':
                    # Grab these named credentials
                    credential_name = value
                    for credential_key, credential_value in self.credentials(credential_name).items():
                        if credential_key != NAME:
                            kwargs[credential_key] = credential_value

            # load and initialize the plugin
            logger.debug(f"Found source '{source[NAME]}'")
            sources.append((source[NAME], self._load_plugin(SOURCE, source['module']), kwargs))

        logger.debug(f"Found {len(sources)} total sources")
        return sources


    def operators(self):
        """Return a list of (name, Operator class, {kwargs}) tuples.

        :raises: threatingestor.exceptions.PluginError
        """
        operators = []
        for operator in self.config['operators']:
            kwargs = {}
            for key, value in operator.items():
                if key not in INTERNAL_OPTIONS:
                    if key == ARTIFACT_TYPES:
                        # parse out special artifact_types option
                        artifact_types = []
                        for artifact in value:
                            try:
                                artifact_types.append(threatingestor.artifacts.STRING_MAP[artifact.lower().strip()])
                            except KeyError:
                                # ignore invalid artifact types
                                pass
                        kwargs[key] = artifact_types

                    elif key == FILTER_STRING:
                        # pass in special filter_string option
                        kwargs['filter_string'] = value

                    elif key == NAME:
                        # exclude name key from operator kwargs, since it's not used
                        pass

                    else:
                        kwargs[key] = value

                elif key == 'credentials':
                    # Grab these named credentials
                    credential_name = value
                    for credential_key, credential_value in self.credentials(credential_name).items():
                        if credential_key != NAME:
                            kwargs[credential_key] = credential_value

            # load and initialize the plugin
            logger.debug(f"Found operator '{operator[NAME]}'")
            operators.append((operator[NAME], self._load_plugin(OPERATOR, operator['module']), kwargs))

        logger.debug(f"Found {len(operators)} total operators")
        return operators

    def whitelists(self):
        """Returns whitelist list."""
        return self.config.get("whitelists", [])


import json


from threatingestor.exceptions import DependencyError
import threatingestor.artifacts
from threatingestor.operators import abstract_json


try:
    import boto3
except ImportError:
    raise DependencyError("Dependency boto3 required for SQS operator is not installed")


class Plugin(abstract_json.AbstractPlugin):
    """Operator for Amazon SQS."""
    def __init__(self, aws_access_key_id, aws_secret_access_key, aws_region, queue_name,
            artifact_types=None, filter_string=None, allowed_sources=None, **kwargs):
        """SQS operator."""
        self.sqs = boto3.client('sqs', region_name=aws_region, aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key)
        self.queue_url = self.sqs.get_queue_url(QueueName=queue_name)['QueueUrl']

        super(Plugin, self).__init__(artifact_types=artifact_types,
                                     filter_string=filter_string,
                                     allowed_sources=allowed_sources,
                                     **kwargs)
        self.artifact_types = artifact_types or [
            threatingestor.artifacts.URL,
        ]


    def _put(self, content):
        """Send content to an SQS queue."""
        return self.sqs.send_message(
                QueueUrl=self.queue_url,
                DelaySeconds=0,
                MessageBody=json.dumps(content)
        )



from __future__ import absolute_import


import re
import requests
import twitter
from loguru import logger
from threatingestor.sources import Source


TWEET_URL = 'https://twitter.com/{user}/status/{id}'

WHITELIST_DOMAINS = r"pastebin\.com"


class Plugin(Source):

    def __init__(self, name, api_key, api_secret_key, access_token, access_token_secret, defanged_only=True, **kwargs):
        self.name = name
        self.api = twitter.Twitter(auth=twitter.OAuth(access_token, access_token_secret, api_key, api_secret_key))

        # Let the user decide whether to include non-obfuscated URLs or not.
        self.include_nonobfuscated = not defanged_only

        # Support for full tweet
        tweet_param = {'tweet_mode': 'extended'}
        kwargs.update(tweet_param)

        # Forward kwargs.
        # NOTE: No validation is done here, so if the config is wrong, expect bad results.
        self.kwargs = kwargs

        # Decide which endpoint to use based on passed arguments.
        # If slug and owner_screen_name, use List API.
        # If screen_name or user_id, use User Timeline API.
        # If q is set, use Search API.
        # Otherwise, default to mentions API.
        self.endpoint = self.api.statuses.mentions_timeline
        if (kwargs.get('slug') and kwargs.get('owner_screen_name')) or (kwargs.get('list_id') and kwargs.get('owner_screen_name')):
            self.endpoint = self.api.lists.statuses
        elif kwargs.get('screen_name') or kwargs.get('user_id'):
            self.endpoint = self.api.statuses.user_timeline
        elif kwargs.get('q'):
            self.endpoint = self.api.search.tweets

    def run(self, saved_state):
        # Modify kwargs to insert since_id.
        if saved_state:
            self.kwargs['since_id'] = saved_state

        # Pull new tweets.
        try:
            response = self.endpoint(**self.kwargs)
        except twitter.api.TwitterHTTPError as e:
            # API error; log and return early.
            logger.warning(f"Twitter API Error: {e}")

            return saved_state, []

        # Correctly handle responses from different endpoints.
        try:
            tweet_list = response['statuses']
        except TypeError:
            tweet_list = response

        tweets = [{
            'content': s.get('full_text', ''),
            'id': s.get('id_str', ''),
            'user': s.get('user', {}).get('screen_name', ''),
            'entities': s.get('entities', {}),
        } for s in tweet_list]

        artifacts = []
        # Traverse in reverse, old to new.
        tweets.reverse()
        for tweet in tweets:

            # Expand t.co links.
            for url in tweet['entities'].get('urls', []):
                try:
                    tweet['content'] = tweet['content'].replace(url['url'], url['expanded_url'])

                    # Check if pastebin.com in url
                    if re.search(WHITELIST_DOMAINS, url['expanded_url']):

                        # Check if the url is already returning the 'raw' pastebin. If not, update the url
                        if 'raw' not in url['expanded_url']:
                            pastebin_id = re.search(r"pastebin.com/(.*?)$", url['expanded_url']).group(1)
                            location = f"https://pastebin.com/raw/{pastebin_id}"
                        else:
                            location = url['expanded_url']

                        req = requests.get(location)
                        saved_state = tweet['id']
                        artifacts += self.process_element(req.text, location, include_nonobfuscated=True)

                        logger.log('NOTIFY', f"Discovered paste: {location}")

                except KeyError:
                    # No url/expanded_url, continue without expanding.
                    pass

        return saved_state, artifacts







import json


from threatingestor.exceptions import DependencyError
from threatingestor.sources import abstract_json


try:
    import greenstalk
except ImportError:
    raise DependencyError("Dependency greenstalk required for Beastalk operator is not installed")


BATCH_SIZE = 10


class Plugin(abstract_json.AbstractPlugin):
    """Source for Beanstalk work queue."""
    def __init__(self, name, host, port, queue_name, paths, reference=None):
        """Beanstalk source."""
        super(Plugin, self).__init__(name, paths, reference)
        self.queue = greenstalk.Client(host, port, watch=queue_name)


    def get_objects(self, saved_state):
        job_list = []

        # Just do one job at a time, so we don't have to worry about batching
        # or TTL timeouts.
        try:
            batch_count = 0
            while batch_count < BATCH_SIZE:
                message = self.queue.reserve(timeout=1)
                job = json.loads(message.body)
                job_list.append(job)

                # Let the queue know that the message is processed.
                self.queue.delete(message)
                batch_count += 1

        except greenstalk.TimedOutError:
            # No more jobs.
            pass

        # Return the job list.
        return saved_state, job_list



from threatingestor.operators import Operator
import threatingestor.artifacts

class AbstractPlugin(Operator):
    """Operator for Abstract JSON"""

    def __init__(self, artifact_types=None, filter_string=None, allowed_sources=None, **kwargs):
        # kwargs are used to dynamically form message body
        self.kwargs = kwargs

        super(AbstractPlugin, self).__init__(artifact_types=artifact_types,
                                     filter_string=filter_string,
                                     allowed_sources=allowed_sources)

        self.artifact_types = artifact_types or [
            threatingestor.artifacts.URL,
        ]

    def handle_artifact(self, artifact):
        """Operate on a single artifact"""
        message_body = dict([(k, artifact.format_message(v)) for (k, v) in self.kwargs.items()])
        self._put(message_body)

    def _put(self,content):
        raise NotImplementedError()



class IngestorError(Exception):
    """Base exception class."""
    pass

class DependencyError(IngestorError):
    """Missing dependency."""
    pass

class PluginError(IngestorError):
    """Missing plugin."""
    pass



from __future__ import absolute_import
import sqlite3


import threatingestor.artifacts
from threatingestor.operators import Operator


class Plugin(Operator):
    """Operator for SQLite3."""
    def __init__(self, filename, artifact_types=None, filter_string=None, allowed_sources=None):
        """SQLite3 operator."""
        super(Plugin, self).__init__(artifact_types, filter_string, allowed_sources)
        self.artifact_types = artifact_types or [
            threatingestor.artifacts.Domain,
            threatingestor.artifacts.Hash,
            threatingestor.artifacts.IPAddress,
            threatingestor.artifacts.URL,
            threatingestor.artifacts.YARASignature,
            threatingestor.artifacts.Task,
        ]

        # Connect to SQL and set up the tables if they aren't already.
        self.sql = sqlite3.connect(filename)
        self.cursor = self.sql.cursor()

        self._create_tables()


    def _create_tables(self):
        """Create tables for each supported artifact type."""
        for artifact_type in self.artifact_types:
            type_name = artifact_type.__name__.lower()
            query = f"""
                CREATE TABLE IF NOT EXISTS `{type_name}` (
                    `artifact` TEXT PRIMARY KEY,
                    `reference_link` TExT,
                    `reference_text` TEXT,
                    `created_date` TEXT,
                    `state` TEXT
                )
            """
            self.cursor.execute(query)
        self.sql.commit()


    def _insert_artifact(self, artifact):
        """Insert the given artifact into its corresponding table."""
        type_name = artifact.__class__.__name__.lower()
        query = f"""
            INSERT OR IGNORE INTO `{type_name}` (
                `artifact`,
                `reference_link`,
                `reference_text`,
                `created_date`,
                `state`
            )
            VALUES (?, ?, ?, datetime('now', 'utc'), NULL)
        """
        self.cursor.execute(query, (
            str(artifact),
            artifact.reference_link,
            artifact.reference_text
        ))
        self.sql.commit()


    def handle_artifact(self, artifact):
        """Operate on a single artifact."""
        self._insert_artifact(artifact)
